# Copyright (c) Facebook, Inc. and its affiliates.
# All rights reserved.

# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.

from PIL import ImageFilter
import random


class TwoCropsTransform:
    """Take two random crops of one image as the query and key."""

    def __init__(self, base_transform):
        self.base_transform = base_transform

    def __call__(self, x):
        q = self.base_transform(x)
        k = self.base_transform(x)
        return [q, k]

class FourCropsTransform:
    """Take two random crops of one image as the query and key."""

    def __init__(self, base_transform, base_transformPJ):
        self.base_transform = base_transform
        self.base_transformPJ = base_transformPJ

    def __call__(self, x):
        q = self.base_transform(x)
        k = self.base_transform(x)
        qPJ = self.base_transformPJ(x)
        kPJ = self.base_transformPJ(x)
        return [q, k, qPJ, kPJ]


class GaussianBlur(object):
    """Gaussian blur augmentation in SimCLR https://arxiv.org/abs/2002.05709"""

    def __init__(self, sigma=[.1, 2.]):
        self.sigma = sigma

    def __call__(self, x):
        sigma = random.uniform(self.sigma[0], self.sigma[1])
        x = x.filter(ImageFilter.GaussianBlur(radius=sigma))
        return x
