import pytorch_lightning as pl
from data.datasets import TinyImageNet
from torch.utils.data import DataLoader


class TinyImageNetDataModule(pl.LightningDataModule):
    def __init__(
        self,
        num_workers: int = 16,
        normalize: bool = False,
        batch_size: int = 32,
        pin_memory: bool = False,
        drop_last: bool = False,
    ):
        super().__init__()

        self.num_workers = num_workers
        self.batch_size = batch_size
        self.pin_memory = pin_memory
        self.drop_last = drop_last
        self.train_transforms = None
        self.val_transforms = None
        self.im_dir = "../datasets/102flowers"
        self.num_classes = 102

    def setup(self, stage=None):
        """
        Creates train, val, and test dataset
        """
        self.train_data = TinyImageNet(
            self.im_dir, mode="train", transform=self.train_transforms
        )
        self.valid_data = TinyImageNet(
            self.im_dir, mode="validation", transform=self.val_transforms
        )

    def train_dataloader(self):

        train_split = DataLoader(
            dataset=self.train_data,
            num_workers=self.num_workers,
            batch_size=self.batch_size,
            shuffle=True,
            pin_memory=True,
            drop_last=True,
        )
        return train_split

    def val_dataloader(self):

        val_split = DataLoader(
            dataset=self.valid_data,
            num_workers=self.num_workers,
            batch_size=self.batch_size,
            shuffle=False,
            pin_memory=False,
            drop_last=False,
        )
        return val_split

    def test_dataloader(self):
        test_split = DataLoader(
            dataset=self.test_data,
            num_workers=self.num_workers,
            batch_size=self.batch_size,
            shuffle=False,
            pin_memory=False,
            drop_last=False,
        )
        return test_split

    @property
    def num_samples(self) -> int:
        return len(self.train_data)
