import re
from argparse import ArgumentParser
from typing import Optional, Tuple

import torch
from torch import nn

from models.squeezenet import squeezenet1_1
import pytorch_lightning as pl
from models.evaluator import SSLEvaluator
from models.resnets import resnet18, resnet50
from models.utils import torchvision_ssl_encoder
from utils.checkpoint_utils import load_pl_ckpt
from utils.weight_initializers import init_weights


class SimCLR(pl.LightningModule):
    def __init__(self, args, **kwargs):

        super().__init__()

        self.arch = arch

        self.first_conv = first_conv
        self.maxpool1 = maxpool1

        self.encoder = self.init_model()

        self.evaluator = nn.Sequential(
            nn.Flatten(), nn.Linear(512, n_classes, bias=False)
        )

    def init_model(self):
        if self.arch == "resnet18":
            backbone = resnet18
        elif self.arch == "resnet50":
            backbone = resnet50

        return backbone(
            first_conv=self.first_conv,
            maxpool1=self.maxpool1,
            return_all_feature_maps=False,
        )

    def forward(self, inputs):

        features = self.encoder(inputs)[-1]

        if not self.fs and not self.train_all:
            features = features.detach()

        ## - classifier for backbone
        outputs = self.evaluator(features)

        return outputs

    def load_pretrained(self, state_dict):

        new_dict = {k: state_dict[k] for k in state_dict if re.match(".*encoder.*", k)}
        new_dict = {re.sub(".*encoder.", "", k): v for k, v in new_dict.items()}
        if bool(new_dict):
            self.encoder.load_state_dict(new_dict)
        else:
            self.encoder.load_state_dict(state_dict)

        if not self.fs and not self.train_all:
            print("\nModel:: encoder parameters freezed.")
            for param in self.encoder.parameters():
                param.requires_grad = False

    @staticmethod
    def add_model_specific_args(parent_parser):
        parser = ArgumentParser(parents=[parent_parser], add_help=False)
        # model params
        parser.add_argument(
            "--arch", default="resnet18", type=str, help="convnet architecture"
        )
        # specify flags to store false
        parser.add_argument("--first_conv", action="store_false")
        parser.add_argument("--maxpool1", action="store_false")

        return parser
