import csv
import os
import random

import numpy as np
import torch
import torch.nn as nn
import torch.optim as optim
import torchvision
import torchvision.transforms as transforms
from torch.utils.data import DataLoader

from data.dataloaders import get_test_data
from models.simsiam import FT_SimSiam
from utils.option_parser import optionParser

if __name__ == "__main__":

    # Training Options

    torch.backends.cudnn.deterministic = True

    OP = optionParser()

    OP.parser = FT_SimSiam.add_model_specific_args(OP.parser)

    args = OP.get_args()

    file_object = open("./results.txt", "a")

    # cuda setup
    cuda_check = False
    if torch.cuda.is_available() and args.device != "cpu":
        cuda_check = True

    # Seed setting for random operations
    seed = 1234
    torch.manual_seed(seed)
    random.seed(seed)
    os.environ["PYTHONHASHSEED"] = str(seed)
    np.random.seed(seed)
    torch.manual_seed(seed)
    torch.backends.cudnn.deterministic = True
    torch.backends.cudnn.benchmark = False
    if cuda_check:
        torch.cuda.manual_seed(seed)
        torch.cuda.manual_seed_all(seed)

    # data loaders
    print("\n===> Loading Dataset")

    args.data_mode = ""

    testloader, n_classes = get_test_data(args)

    # print("\nAugmentations:\n\t"+ testloader.dataset.trs)

    # add model specific args.
    print("\n==> Model initialization")

    model = FT_SimSiam(args, n_classes)

    # checkpoint loading
    if args.ckpt != None:
        print("\n==> Resuming from checkpoint")
        checkpoint = torch.load(
            args.ckpt, map_location=lambda storage, loc: storage)
        import ipdb
        ipdb.set_trace()
        model.load_state_dict(checkpoint["model_state_dict"])
        # checkpoint = torch.load(
        #     args.ckpt, map_location=lambda storage, loc: storage)

        # import ipdb
        # ipdb.set_trace()

        # if "model_state_dict" in checkpoint:
        #     model.load_pretrained(checkpoint["model_state_dict"])
        # elif "state_dict" in checkpoint:
        #     model.load_pretrained(checkpoint["state_dict"])
        # else:
        #     model.load_pretrained(checkpoint)
        # if not args.from_scratch and not args.train_all:
        #     print("\nEncoder on eval.")
        #     model.encoder.eval()

    # GPU setting

    print("\n===> Setting GPU")

    if cuda_check:
        model.to(args.device)

    # Training Loop

    print("\n===> Testing\n")

    correct = 0
    total = 0
    with torch.no_grad():
        model.eval()
        for data in testloader:
            images, labels = data
            if cuda_check:
                images = images.to(args.device)
                labels = labels.to(args.device)

            outputs = model(images)

            _, predicted = torch.max(outputs.data, 1)

            total += labels.size(0)
            correct += (predicted == labels).sum().item()

    accuracy = 100 * correct / total

    file_object.write(str(accuracy) + "\n")
    file_object.close()
    print("Accuracy: {:.2f}%".format(accuracy))
