import argparse

import torch


class optionParser(object):
    def __init__(self):

        self.parser = argparse.ArgumentParser(argument_default=argparse.SUPPRESS)
        self.parser.add_argument(
            "--device",
            type=str,
            default="cuda:0",
            help="Device for training [cpu, cuda:0, cuda:1]",
        )

        self.parser.add_argument(
            "--ckpt", type=str, default=None, help="checkpoint to restore"
        )

        self.parser.add_argument(
            "--im_dir", type=str, default=None, help="directory containing input images"
        )

        self.parser.add_argument(
            "--save_dir",
            type=str,
            default="./experiments/",
            help="where to save experiments",
        )

        self.parser.add_argument(
            "--dataset", type=str, default="102flowers", help="dataset to use"
        )


        self.parser.add_argument(
            "--model", type=str, default="simsiam", help="contrastive model"
        )
        self.parser.add_argument("--dim", type=int, default=32, help="image dimension")
        self.parser.add_argument(
            "--epochs", type=int, default=200, help="Number of training epochs"
        )
        self.parser.add_argument(
            "--pj_idx", type=int, default=None, help="index of plankian jitter"
        )
        self.parser.add_argument(
            "--lr", type=float, default=1e-3, help="starting learning rate"
        )
        self.parser.add_argument(
            "--weight_decay", type=float, default=5e-4, help="starting learning rate"
        )
        self.parser.add_argument(
            "--batch_size", type=int, default=128, help="size of the training batch"
        )

        self.parser.add_argument(
            "--threads",
            type=int,
            default=4,
            help="number of threads for data loader to use",
        )

        self.parser.add_argument(
            "--from_scratch",
            action="store_true",
            default=False,
            help="Train from scratch or finetune?",
        )

        self.parser.add_argument(
            "--test", action="store_true", default=False, help="Test mode"
        )
        self.parser.add_argument(
            "--pj_analysis", action="store_true", default=False, help="Test mode"
        )

        self.parser.add_argument(
            "--train_all",
            action="store_true",
            default=False,
            help="Train all or only linear?",
        )

        self.parser.add_argument(
            "--scheduler",
            action="store_true",
            default=False,
            help="Activare sosine LR scheduler",
        )

        self.parser.add_argument(
            "--optim", type=str, default="sgd", help="optimizer to use [sgd, adam]"
        )

        self.parser.add_argument(
            "--base_db", type=str, default="", help="db used for pretraining"
        )

        self.args = None

    def get_parser(self):
        return self.parser

    def get_args(self):
        self.args = self.parser.parse_args()

        ### GPU board availability check
        self.args.cuda = False
        if torch.cuda.is_available() and self.args.device != "cpu":
            self.args.cuda = True

        return self.args

    def print_args(self):

        print(
            "\n===> parserions \
                \n \
                \nModel checkpoint: {} \
                \nGPU: {}".format(
                self.args.ckpt, self.args.device
            )
        )
